#include <gpstdlib.h>
#include <gpstdio.h>
#include <gpgraphic.h>
#include <gpgraphic16.h>

#include "gpmain.h"
#include "../include/global.h"
#include "../include/graphics16.h"
#include "../include/okf.h"
#include "../include/fileSelector.h"

#include "project.h"
#include "fonts.h"
#include "background.h"


static void Initialize(void)
{
  InitGraphics(16);
  giSurface = 1;
//  SetClockSpeed(CLOCK40);
  GpFatInit();

  OkfInitialize(gtSurface, &giSurface);
  OkfRegister(verdana10_okf);
  OkfRegister(verdana8b_p_okf);
  OkfRegister(comicSansMS12b_pm_okf);
  OkfClone(3);
  OkfApplyEffect(okf.pFonts->lastHandle, okfOverlay, RGB(31, 31, 31), 15);
  OkfClone(3);
  OkfApplyEffect(okf.pFonts->lastHandle, okfOverlay, RGB(15, 31, 15), 15);
  OkfClone(3);
  OkfGetFont(okf.pFonts->lastHandle)->pOkf->alphaMask = FALSE;

  // Initialize graphics.h alpha blending look up table
  gtAlphaLUT = okf.alphaLUT.pLUT;
  giAlphaLevels = okf.alphaLUT.levels;
}

static void Terminate(void)
{
  OkfTerminate();
}


static void RefreshScreen(int currentOption)
{
  OkfSetToDefaults();
  okf.font = 2;
  okf.effect.color = RGB(31, 15, 0);

  // Background
  GpRectFill16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT, RGB(7, 15, 7));

  okf.justify = okfCenter;

  // Title
  OkfPrintAt(GPC_LCD_WIDTH / 2, 30, "FILE SELECTOR TESTER\n\n\n");

  // Menu options
  okf.effect.type = (currentOption == 1) ? okfAddColor : okfNone;
  OkfPrint("Default FileSelector\n\n");
  
  okf.effect.type = (currentOption == 2) ? okfAddColor : okfNone;
  OkfPrint("Custom FileSelector\n\n");
  
  okf.effect.type = (currentOption == 3) ? okfAddColor : okfNone;
  OkfPrint("Exit");

  okf.justify = okfJustifyLeft;
  okf.effect.type = okfNone;

  // Memory
  OkfPrintAt(12, 193, "Memory: ");
  OkfPrintLong(gm_availablesize(), "%ld");

  // Copyright
  GpRectFill16(NULL, &gtSurface[giSurface], 0, 215, GPC_LCD_WIDTH, 1, RGB(31, 31, 31));
  OkfPrintAt(12, 220, AUTHOR);
  OkfPrintAt(170, okf.y, COPYRIGHT);

  SurfaceFlip();
}

static void MainMenu(void)
{
  FILESELECTOR fileSelector;
  int tick;
  int pressedKeys, newKeys, oldKeys;
  int currentOption = 1;

  while(TRUE)
  {
    tick = GpTickCountGet();
    
    // Refresh screen
    RefreshScreen(currentOption);

    // Get buttons state
    oldKeys = pressedKeys;
    newKeys = pressedKeys = GpKeyGet();

    // Filter buttons
    if ((newKeys & GPC_VK_UP) && (oldKeys & GPC_VK_UP))
      newKeys = newKeys & ~GPC_VK_UP;
    if ((newKeys & GPC_VK_DOWN) && (oldKeys & GPC_VK_DOWN))
      newKeys = newKeys & ~GPC_VK_DOWN;
    if ((newKeys & GPC_VK_FA) && (oldKeys & GPC_VK_FA))
      newKeys = newKeys & ~GPC_VK_FA;

    // Manage buttons
    if (newKeys & GPC_VK_UP)
    {
      if (currentOption > 1)
        currentOption--;
    }
    else if (newKeys & GPC_VK_DOWN)
    {
      if (currentOption < 3)
        currentOption++;
    }
    else if (newKeys & GPC_VK_FA)
    {
      switch (currentOption)
      {
        case 1: // default FileSelector
        case 2: // custom FileSelector
          FileSelectorInit(&fileSelector);

          if (currentOption == 2)
          {
            fileSelector.pFileSizeFormat = " %s";
            fileSelector.pTotalsFormat = "%s bytes, %s files";

            fileSelector.pathLeft = 12;
            fileSelector.pathTop  = 6;
            fileSelector.listTop  = 57;
            fileSelector.listSize = 6;
            fileSelector.fileLeft = 51;
            fileSelector.fileYDisplacement = -3;
            fileSelector.sizeRight = 290;
            fileSelector.sizeYDisplacement = -2;
            fileSelector.totalsLeft = 13;
            fileSelector.totalsTop = 207;
            fileSelector.scrollBarLeft = 18;
            fileSelector.selectionHeight = 21;
            fileSelector.selectionMargin = 15;

            fileSelector.showExtensions = FALSE;
            fileSelector.showSeparationLines = FALSE;

            fileSelector.pathFont = 5;
            fileSelector.folderFont = 4;
            fileSelector.fileFont = 3;
            fileSelector.sizesFont = 6;
            fileSelector.totalsFont = 5;

            fileSelector.pBackground = (unsigned short *) background;
            fileSelector.backColor = RGB(0, 0, 0);
            fileSelector.selectionColor = RGB(31, 31, 31);
            fileSelector.selectionIntensity = 16;
            fileSelector.scrollBarColor = RGB(31, 31, 31);
          }

          while (FileSelector(&fileSelector) == 1)
          {
            GpRectFill16(NULL, &gtSurface[giSurface], 0, 0, GPC_LCD_WIDTH, GPC_LCD_HEIGHT, RGB(7, 15, 7));
            okf.font = 2;
            OkfPrintAt(10, 10, fileSelector.filename);
            SurfaceFlip();
            while (GpKeyGet());
            while (!GpKeyGet());
            while (GpKeyGet());
          }
          break;

        case 3: // exit
          return;
      }
    }
    
    while ((GpTickCountGet() - tick) < 50);
  }
}

void GpMain (void * arg)
{
  Initialize();
  MainMenu();
  Terminate();
  GpAppExit();
}
